/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package player;

import graphics.StandardVisualizationData;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JPanel;
import network.Constraint;
import network.LinkMessage;
import network.Network;
import network.NetworkConstraint;
import thread.LinksThread;
import thread.PaintingThread;

/**
 *
 * @author Dur
 */
public class Player {
  public static final int STATUS_STOPPED=0;
  public static final int STATUS_STARTED=1;
  public static final int STATUS_PAUSED=2;
  public static final int MESSAGE_STOP=0;
  public static final int MESSAGE_START=1;
  public static final int MESSAGE_PAUSE=2;
  public static final int MESSAGE_WAKEUP=3;
  
  protected int fps=25;
  protected int paintingRefreshTime;
  protected int status=STATUS_STOPPED;
  protected float actualFps=0;
  protected long totalRunningTime;
  protected long startedRunningTime;
  protected StringBuffer errorLog=new StringBuffer();
  protected int errorCount=0;
  protected StringBuffer assertLog=new StringBuffer();
  protected int assertCount=0;
  
  public Network network;
  public JPanel screen;
  public LinksThread linksThread=null;
  public PaintingThread paintingThread=null;
  
  public Player(Network network, JPanel screen)
  {
    this.network=network;
    this.screen=screen;
    SetFPS(fps);
    Constraint.Initialize(this);
  }
  
  public void Start()
  {
    if (status==STATUS_STOPPED)
    {
      status=STATUS_STARTED;
      errorLog=new StringBuffer();
      errorCount=0;
      assertLog=new StringBuffer();
      assertCount=0;
      network.PrepareForVisualization(this);
      linksThread=new LinksThread(this);
      paintingThread=new PaintingThread(this);
      
      totalRunningTime=0;
      startedRunningTime=new Date().getTime();
      
      linksThread.start();
      paintingThread.start();
      for (int i=0;i<network.nodes.length;i++)
      {
        network.nodes[i].nodeThread.start();
      }
    }
    
    if (status==STATUS_PAUSED)
    {
      status=STATUS_STARTED;
      startedRunningTime=new Date().getTime();
      
      linksThread.PutObjectFirst(MESSAGE_START);
      paintingThread.PutObjectFirst(MESSAGE_START);
      for (int i=0;i<network.nodes.length;i++)
      {
        network.nodes[i].nodeThread.PutObjectFirst(MESSAGE_START);
      }
    }
  }
  
  public void Pause()
  {
    if (status==STATUS_PAUSED)
    {
      Start();
      return;
    }
    if (status==STATUS_STARTED)
    {
      for (int i=0;i<network.nodes.length;i++)
      {
        network.nodes[i].nodeThread.PutObjectFirst(MESSAGE_PAUSE);
      }
      linksThread.PutObjectFirst(MESSAGE_PAUSE);
      paintingThread.PutObjectFirst(MESSAGE_PAUSE);
      totalRunningTime+=new Date().getTime()-startedRunningTime;
      status=STATUS_PAUSED;
    }
  }
  
  public void Stop()
  {
    if ((status==STATUS_STARTED) || (status==STATUS_PAUSED))
    {
      for (int i=0;i<network.nodes.length;i++)
      {
        network.nodes[i].nodeThread.PutObjectFirst(MESSAGE_STOP);
        network.nodes[i].nodeThread.stop();
      }
      linksThread.PutObjectFirst(MESSAGE_STOP);
      linksThread.stop();
      paintingThread.PutObjectFirst(MESSAGE_STOP);
      paintingThread.stop();
      status=STATUS_STOPPED;
      totalRunningTime+=new Date().getTime()-startedRunningTime;
    }
  }
  
  public int GetStatus()
  {
    return status;
  }
  
  public void SetFPS(int fps)
  {
    this.fps=fps;
    paintingRefreshTime=Math.round(1000/((float)fps));
  }
  
  public int GetPaintingRefreshTime()
  {
    return paintingRefreshTime;
  }
  
  public int GetFPS()
  {
    return fps;
  }
  
  public long GetRunningTime()
  {
    if (status==STATUS_STARTED)
    {
      return totalRunningTime+(new Date().getTime()-startedRunningTime);
    }
    return totalRunningTime;
  }
  
  public int GetSentMessageCount()
  {
    return NetworkConstraint.numSentMessages;
  }
  
  public int GetActualMessageCount()
  {
    return NetworkConstraint.numActualMessages;
  }
  
  public LinkMessage GetMessageAtPosition(int posX, int posY)
  {
    if (linksThread!=null)
    {
      StandardVisualizationData visualizationData=network.GetAlgorithm().GetActivatedVisualizationData();
      synchronized (linksThread.activeMessages)
      {
        Iterator<LinkedList<LinkMessage>> it=linksThread.activeMessages.values().iterator();
        while (it.hasNext())
        {
          Iterator<LinkMessage> it2=it.next().iterator();
          while (it2.hasNext())
          {
            LinkMessage mes=it2.next();
            if (!visualizationData.notVisualizingMessageTypes.containsKey(mes.message.GetType()))
            {
              float ratio=mes.GetProgress();
              if (ratio>1)
                ratio=1;
              int x,y;
              x=mes.recipient.position.x-mes.sender.position.x;
              x=mes.sender.position.x+Math.round(((float)x)*ratio);
              y=mes.recipient.position.y-mes.sender.position.y;
              y=mes.sender.position.y+Math.round(((float)y)*ratio);
              int dist=(int)Math.sqrt((posX-x)*(posX-x)+(posY-y)*(posY-y));
              if (dist<=LinkMessage.dimension.x)
              {
                return mes;
              }
            }
          }
        }
      }
    }
    return null;
  }
  
  public void WriteException(String desc, Exception e)
  {
    synchronized (errorLog)
    {
      errorCount++;
      errorLog.append(desc+" ("+e.getMessage()+"): ");
      if (e.getStackTrace().length>0)
      {
        String s=e.getStackTrace()[0].getMethodName()+": line "+e.getStackTrace()[0].getLineNumber();
        errorLog.append(s);
      }
      errorLog.append("\n");
    }
  }
  
  public String GetExceptions()
  {
    String s="";
    synchronized (errorLog)
    {
      s=errorLog.toString();
    }
    return s;
  }
  
  public int GetExceptionCount()
  {
    return errorCount;
  }
  
  public void WriteAssert(String text)
  {
    if (text!=null)
    {
      synchronized (assertLog)
      {
        assertCount++;
        assertLog.append(text);
        assertLog.append("\n");
      }
    }
  }
  
  public String GetAssert()
  {
    String s="";
    synchronized (assertLog)
    {
      s=assertLog.toString();
    }
    return s;
  }
  
  public int GetAssertCount()
  {
    return assertCount;
  }
  
}
